//*****************************************************************************
//      C++ functions for:
//		arcball driver
//
//      Written by Stuart Prescott:
//              Written:                21/08/1996
//              Last Modified:          27/10/1996
//
//      Libraries Required:
//              libsx (Dominic Giamapolo - Athena Widget Set)
//
//      System headers:
//              Streamed I/O
//              Maths fns
//
//      Other Object Classes used:
//              3D points / vectors / matrices
//              ArcBall
//              colours
//              libx-extensions
//
//*****************************************************************************
  
  
extern "C" {  
#include <libsx.h>  
}  
  
#include <iostream.h>  
#include <math.h>
  
#include "vectors.h"  
#include "arcball.h"  
#include "coldefs.h"  
#include "grafxten.h"
  
#define DrawAreaHeight  150  
#define DrawAreaWidth   150  
#define pi 		3.14159265  

extern ColourDefs *pal;
  
// *************************************************************************
// *************************************************************************
ArcballWin::ArcballWin(Widget parent, int where1, Widget from1,  
				      int where2, Widget from2, MyProgram *data)
{  

    AllData = data; 
 
/*      widget = MakeForm(parent, where1, from1,  
			      where2, from2);  
*/  
  
    SetForm(parent);  
  
    // set up a draw area to display the colour  
    draw_area = MakeDrawArea (DrawAreaWidth, DrawAreaHeight,  
                                ArcBallWinRedisplay, this);  
 
    SetWidgetPos(draw_area, where1, from1, where2, from2);  
  
    widget=draw_area;  
    // show the window  
    //ShowDisplay();  
    SetButtonDownCB (draw_area, ArcballMouseDown);  
  
    //set up the screen positions of the world co-ordinates  
  
    setscreen( point3(-1, 1), point3(1, -1) );  
    setpixels(DrawAreaWidth, DrawAreaHeight);  
  
    ResetMatrix();

    temp1 = matrix (1, 0, 0,
		    0, 1, 0,
		    0, 0, 1); 
    temp2=temp1;
  
}
  
// *************************************************************************
// *************************************************************************
void ArcballWin::ResetMatrix()  
{
    double theta= 54.74 * pi / 180.0;  
    double phi=   45.0   * pi / 180.0;  
    double ct= cos (theta);  
    double st= sin (theta);  
    double cp= cos (phi);  
    double sp= sin (phi);  
  
    m = matrix (-sp,     cp,     0,   
	        -ct*cp, -ct*sp,  st,   
	         st*cp,  st*sp,  ct  ); 
    return; 
} 
  
// *************************************************************************
// *************************************************************************
void  
ArcBallWinRedisplay(Widget w, int width, int height, void *data)  
{  
    //Set up the black display with the circle

    SetBgColor(w, pal->black);
    SetColor(pal->white);
    DrawBox(0,0, width-1, height-1);
    ClearDrawArea();  
    SetColor(pal->white);
    DrawArc (0,0, DrawAreaWidth-1, DrawAreaHeight-1, 0, 360);  
  
}  
  
// *************************************************************************
// *************************************************************************
void  
ArcballMouseDown (Widget w, int which_button, int x, int y, void *data)  
{  
    if (which_button != 1)  
    {  
        return;  
    }  
 
    ArcballWin *me1 = (ArcballWin *)data;
 
    me1->Xstart.x = x;  
    me1->Xstart.y = y;  
  
    me1->vstart = me1->toworld(me1->Xstart);  
    tosphere (me1->vstart);  
  
    me1->vcurrent = me1->vstart;  
    me1->vcurrent.x -= 0.01;		    //fudge out the zero vectors  
  
    SetButtonDownCB (me1->draw_area, NULL);  
    SetMouseMotionCB (me1->draw_area, ArcballMouseMove);  
    SetButtonUpCB (me1->draw_area, ArcballMouseUp);  
  
  
    // UNDRAW all  
    ClearDrawArea();  
  
    // DRAW the arcball  
    me1->DrawArcBall();  
}  
  
  
// *************************************************************************
// *************************************************************************
void  
ArcballMouseMove (Widget w, int x, int y, void *data)  
{  
    ArcballWin *me1 = (ArcballWin *)data;  
  
    me1->Xcurrent.x = x;  
    me1->Xcurrent.y = y;  
  
    me1->vcurrent = me1->toworld(me1->Xcurrent);  
  
    if (me1->vstart == me1->vcurrent)  
    {  
        me1->vcurrent.x -= 0.01;        //fudge out the zero vectors  
    }  
  
    tosphere (me1->vcurrent);  
  
    //cout << "undraw all . . . ";  
      
    // UNDRAW all  
    ClearDrawArea();  
  
    //cout << "draw arcball\n";  
    // DRAW the arcball and arc  
    me1->DrawArcBall();  
    me1->DrawArcBallArc();  
  
}  
  
// *************************************************************************
// *************************************************************************
void  
ArcballMouseUp (Widget w, int which_button, int x, int y, void *data)  
{  
    if (which_button == 1)  
    {  
	//cout << "Button1 Up\n";  
	ArcballWin *me1 = (ArcballWin *)data;  
  
	SetMouseMotionCB (me1->draw_area, NULL);  
	SetButtonUpCB (me1->draw_area, NULL);  
	SetButtonDownCB (me1->draw_area, ArcballMouseDown);  
  
	SetDrawArea(me1->AllData->mainwin.draw_area);
	me1->m = me1->temp1 * me1->temp2 * me1->m; 
 
	Redisplay(me1->AllData->mainwin.draw_area, 
		  me1->AllData->mainwin.width, me1->AllData->mainwin.height,
		  me1->AllData);

        SetDrawArea(w);
        //ClearDrawArea();
	ArcBallWinRedisplay(w, 0,0, data);
    }  
}  
  
  
// *************************************************************************
// *************************************************************************
// OTHER FUNCTIONS used by the callbacks:  
  
void  
ArcballWin::DrawArcBall() 
{  
    //vector xi, is the vector of mouse down (vstart); xj is current  
    //vector z will be axis of rotation  
    //vectors yi and yj are used for the matrix calculations  
  
    const vector xi (vstart);  
    const vector xj (vcurrent);  
  
  
    vector z  ( (xi.cross(xj)).unit() );  
    vector yi ( z.cross(xi) );  
    vector yj ( z.cross(xj) );  
  
    ::DrawArc (0,0, DrawAreaWidth-1, DrawAreaHeight-1, 0, 360);  
  
  
    double stem = 0.8;			    // length of the stem to cone  
    double srad = 0.1;			    // radius of the small cone  
    double lrad = 0.15;			    // radius of the large cone  
  
    DrawLine( toscreen( point3(0,0) ),   
	      toscreen( point3(xi * stem) ) );  
    DrawCone (xi, stem, srad, this);  
  
    DrawLine( toscreen( point3(0,0) ),  
	      toscreen( point3(yi * stem) ) );  
    DrawCone (yi, stem, srad, this);  
  
    DrawCone( z, 0.0, lrad, this);  

    temp1.setcol(1, xj);  
    temp1.setcol(2, yj);  
    temp1.setcol(3, z );  
  
    temp2.setrow(1, xi);  
    temp2.setrow(2, yi);  
    temp2.setrow(3, z );  
  
}  
  
// *************************************************************************
// *************************************************************************
void  
ArcballWin::DrawArcBallArc()  
{  
    const int N=10;	        //the num of line segments for the arc
    XPoint P[N+1];		//a list of vectors to be sent to polyline
 
    GetArc (vstart, vcurrent, vector(0,0,0),
	    P, N, this);  
  
  
    DrawPolyline (P, (N+1));
  
}  
  
// *************************************************************************
// *************************************************************************
void  
GetArc	 (const vector& p0, const vector& p1, const vector& centre,   
	  XPoint* XP, int N, ScreenInfo *me1)  
{  
    vector *P;  
    P= new vector[N+1];		//a list of vectors to be sent to polyline  
    P[0] = (p0 - centre);  
    P[N] = p1 - centre;  
      
    //calculate the first point   
    double omega = acos( P[0].dot( P[N] ) );  
    P[1] =(P[0] * sin((N-1) * omega/N) + P[N] * sin(omega / N)) / sin(omega);  
  
  
    // use reflections to find the rest (ie points 1..(N-1) )  
    int c;  
    for (c = 2; c < N ;  ++c)  
    {  
        P[c] = 2 * (P[c-2].dot(P[c-1])) * P[c-1] - P[c-2];  
    }  
  
    // have to convert the vectors to XPoints in screen co-ords //OUCH!!!  
    for (c = 0; c <= N;  ++c)  
    {  
        XP[c] = me1->toscreen( point3(P[c] + centre));  
    }  
  
}  
  
// *************************************************************************
// *************************************************************************
void  
DrawCone (const vector& dir, const double& len, const double& r, ScreenInfo* me1)
{ 
    // this function does not draw the back of the cone correctly . . .
    // the maths starts getting a bit ugly, and cones on the arcball
    // are a low priority :-)
 
    const int arcN=6;			//the num of line segments for the arc  
    const int totN= 2 * arcN + 3;  
    XPoint P[totN];		//a list of vectors to be sent to polyline  
  
    //vector deltaZ (0.0, 0.0, 0.1);  
  
    vector vradperp ( dir.cross(vector(0,1,0)) );  
    vector vrad ( (dir.cross(vradperp).unit() * r ) );  
  
    vector pos ( (dir * len) );  
  
    vector varcstart ( (pos + vrad + (vradperp * 0.01)) );  
    vector varcend   ( (pos - vrad) );  
  
    //P[0] is start of arc  
    //P[0..arcN] are around 1st arc  
    GetArc (varcstart, varcend, pos, P, arcN, me1);  
  
    //P[arcN+1] is point of cone  
    P[arcN+1] = me1->toscreen( point3(dir.unit()) );  
  
  
    //P[arcN+2] is (P[0] - delta) & start of 2nd arc (if required)  
    P[arcN+2] = P[0];  
  
    // 2nd arc only required if it is visible  
    if (pos.dot(vector(0,0,1)) < 0)  
    {  
	//GetArc (varcstart, varcend, pos, P, arcN, me1);  
	//DrawPolyline (P, (totN));  
  
	DrawPolyline (P, (arcN+3));  
    }  
    else  
    {  
	DrawPolyline (P, (arcN+3));  
    }  
  
}  
  
  
  
