//*****************************************************************************
//      Header file for:
//		ArcBall driver and output routines.
//		The ArcBall concept was developed by Ken Shoemake, 
//			Computer Graphics Laboratory,
//			University of Pennsylvania
//
//      Written by Stuart Prescott:
//              Written:                21/08/1996
//              Last Modified:          27/10/1996
//
//      Libraries Required:
//              libsx (Dominic Giamapolo - Athena Widget Set)
//
//      System headers:
//              Maths fns
//
//      Other Object Classes used:
//              win (base class)
//              screen/real conversion
//              3D points / vectors / matrices
//              main program data structure
//
//*****************************************************************************

#ifndef ARCBALL_DEFS_H
#define ARCBALL_DEFS_H 1

#include "win.h"
#include "vectors.h"
#include "scrninfo.h"
//#include "main.h"

// Class definitions
// ****************************************************************************
class ArcballWin : public Win , public ScreenInfo
{
    private:
        //points and vectors
        XPoint Xstart, Xcurrent;        //the SCREEN points of mouse movements
        vector vstart, vcurrent;        //the WORLD points of mouse movements

        matrix m, temp1, temp2;

        Widget draw_area;               //where to put the arcball
        Widget widget;                  //may be used for a form if need be

        struct MyProgram *AllData;             //not pretty, but it works :-)
                                        //keep it as a private member

    public:
        ArcballWin(Widget parent, int where1, Widget from1,
                                  int where2, Widget from2, MyProgram *data);

        matrix* OutputLoc() {return &m;}
	Widget  Where()	{return widget;}

        void ResetMatrix();		//reset matrix back to vector(1,1,1)

	void DrawArcBall();
	void DrawArcBallArc();

	friend void ArcballMouseDown (Widget w, int which_button, 
					int x, int y, void *data);
        friend void ArcballMouseUp   (Widget w, int which_button, 
					int x, int y, void *data);
        friend void ArcballMouseMove (Widget w, 
					int x, int y, void *data);


};    //arcball

// ****************************************************************************
//other functions used for the arcball . . . NOTHING SHOULD CALL THESE!!!!

// the draw_area redisplay fn.
void ArcBallWinRedisplay(Widget w, int new_width, int new_height, void *data);  
  
// prototypes for _other_ functions used by the callbacks:  
void DrawCone	 (const vector& dir, const double& len, const double& r, 
			ScreenInfo* );  
  
void GetArc	 (const vector&, const vector&, const vector&,   
		  	XPoint*, int, ScreenInfo*);  
  
// ****************************************************************************
// other inline functions:

inline void tosphere  (vector& v)
{
    double zsq = 1.0 - (v.x * v.x) - (v.y * v.y);
    if (zsq > 0)
    {
        v.z = sqrt(zsq);
    }
    else
    {
        v.z =0;
        v = v.unit();
    }
}

#include "main.h"

#endif  
