//*****************************************************************************
//      Header file for:
//	    	graph classes
//
//      Written by Stuart Prescott:
//              Written:                20/09/1996
//              Last Modified:          24/10/1996
//
//      Libraries Required:
//              libsx (Dominic Giamapolo - Athena Widget Set)
//
//      System headers:
//              Streamed I/O
//              Formatted I/O
//              Maths fns
//
//      Other Object Classes used:
//              template arrays
//              maths extensions
//
//*****************************************************************************

#ifndef GRAPH_DEFINITIONS_H 
#define GRAPH_DEFINITIONS_H 1 

extern "C" {
    #include <libsx.h>
}
#include "arraytem.h"
#include "mathxtra.h"
 
class Graph  
{  
    private:  
  
        int xend, yend, xp1, yp1, xp2, yp2;  
        double xmin, ymin, xmax, ymax, xpsc, ypsc;  
        int xorig, yorig;  
	int xaxlen, yaxlen;  
        int xtic, ytic;  
        int col, bcol;  
  
	int xpix, ypix;		    //these are not properties of the plot,  
				    //but eliminate some globals . . .  
	Widget draw_area;  
  
  
        char *xlab, *ylab, *title;  
        // booleans to control the plot  
        int xlabdef, ylabdef, titledef;         //whether strings are defined  
        int open;                               //whether plot is drawn  
        int bdr;                                //whether plot has a border  

	int getoriginy() const
	    {return INT( ((ymin + ymax)*double(yorig) - ymin*double(yaxlen))
			    /(ymax+ymin)	);}

    public:  
        Graph() {xlabdef  = FALSE;  
                 ylabdef  = FALSE;  
                 titledef = FALSE;  
                 open     = FALSE;  
                 bdr      = TRUE;}  
        ~Graph() {if (xlabdef)  delete[] xlab;  
                  if (ylabdef)  delete[] ylab;  
                  if (titledef) delete[] title;}  
   
        void setdrawarea (Widget w)  
            {draw_area =w; return;}  
        void setscreensize (int a, int b)  
            {xpix = a; ypix = b; return;}  
        void setdomain (const double& a, const double& b)  
            {xmin = a; xmax = b; return;}  
        void setrange (const double& a, const double& b)  
            {ymin = a; ymax = b; return;}  
        void setcorner (int a, int b)  
            {xp1=a; yp1=b; return;}   
        void setaxislen (int a, int b)  
            {xaxlen=a; yaxlen=b; return;}  
        void setaxistitles (char* X, char* Y)  
            {
	     char *s;  
             if (xlabdef)  delete[] xlab;  
             xlab = new char[strlen(X)+1];  
	     s = xlab; while ((*s++=*X++));  
             if (ylabdef)  delete[] ylab;  
             ylab = new char[strlen(Y)+1];  
	     s = ylab; while ((*s++=*Y++));  
             return;  
	    }  
        void setplottitle (char* T)  
            {
             char *s;  
             if (titledef)  delete[] xlab;  
             title = new char[strlen(T)+1];  
	     s = title; while ((*s++=*T++));  
             return;}  
        void setticlen (int a, int b)  
            {xtic=a; ytic=b; return;}  
        void setborder (int a)  
            {bdr=a; return;}  
        void setcolor (int fg, int bg)  
            {col = fg; bcol = bg; return;}  
  
        void drawaxes ();  
        void eraseaxes();  
        void curve(const doublearray1& x, const doublearray1& y, int fgcol);  
	void marklevel (double val, int fgcol);  
  
        double getxmin() const {return xmin;}  
        double getymin() const {return ymin;}  
        double getxmax() const {return xmax;}  
        double getymax() const {return ymax;}  
  	
	int getblx() const {return xp1;}
        int getbly() const {return yp1;}
        int gettrx() const {return xp2;}
        int gettry() const {return yp2;}

	double getfnval(int y) const 
	    {return( (ymax*double(yorig-y) + ymin*double(y-yorig+yaxlen))
			    / double(yaxlen)	); }
};    //Graph  
 
#endif 
