//*****************************************************************************
//      Header file for:
//		mathematical extensions
//
//      Written by Stuart Prescott:
//              Written:                18/08/1996
//              Last Modified:          27/10/1996
//
//      System headers:
//              Streamed I/O
//		Standard library 
//              Maths fns
//
//*****************************************************************************
  
#ifndef MATHS_EXTRA_DEFINITIONS_H  
#define MATHS_EXTRA_DEFINITIONS_H 1  
  
#include <stdlib.h> 
#include <iostream.h> 
#include <math.h>  

// a safety catch on the factorial function
#define SAFEFACTORIAL	0

  
#define abs(X)	    ((X<0) ? (-(X)) : (X))  
#define MAX(X,Y)    ((X<Y) ? (Y) : (X))  
#define MIN(X,Y)    ((X<Y) ? (X) : (Y))  
#define INT(X)	    (int(rint(X)))  
  
inline double fact(int x)  
{  
    #if SAFEFACTORIAL
	if (x<0)
	{
	    cerr << "-ve value passed to factorial function\n";
	    return(0);
	}
    #endif
    
    switch (x)  
    {  
	case 0:	    return 1;  
	case 1:	    return 1;  
	case 2:	    return 2;  
	case 3:	    return 6;  
	case 4:	    return 24;  
	case 5:	    return 720;  
	case 6:	    return 5040;  
	case 7:	    return 40320;  
	case 8:	    return 362880;  
	case 9:	    return 3628800;  
	case 10:    return 39916800;  
	case 11:    return 479001600;  
	default:    return x*fact(x-1);  
    }  
}  
  
//inline double max (double a, ...)  
inline double max (const double& a1, const double& a2, const double& a3,  
		   const double& a4, const double& a5, const double& a6,  
		   const double& a7)  
{   //man varargs //7  
    double temp = MAX(a1,a2);  
    temp = MAX(temp,a3);  
    temp = MAX(temp,a4);  
    temp = MAX(temp,a5);  
    temp = MAX(temp,a6);  
    temp = MAX(temp,a7);  
      
    return (temp);  
  
  
//    return (MAX(a1,(MAX(a2, MAX(a3, MAX(a4, MAX(a5, MAX(a6, a7))))))));  
}  
  
//inline double min (double a,...)  
inline double min (const double& a1, const double& a2, const double& a3,  
		   const double& a4, const double& a5, const double& a6,  
		   const double& a7)  
{   //man varargs //7  
  
    double temp = MIN(a1,a2);  
    temp = MIN(temp,a3);  
    temp = MIN(temp,a4);  
    temp = MIN(temp,a5);  
    temp = MIN(temp,a6);  
    temp = MIN(temp,a7);  
      
    return (temp);  
  
//    return (MIN(a1,(MIN(a2, MIN(a3, MIN(a4, MIN(a5, MIN(a6, a7))))))));  
}  
  
//inline int max (int a, ...)  
inline int max (const int& a1, const int& a2, const int& a3,  
		const int& a4)  
{   //man varargs //4  
    int temp = MAX(a1,a2);  
    temp = MAX(temp,a3);  
    temp = MAX(temp,a4);  
     
    return (temp);  
  
//    return (MIN(a1,(MIN(a2, MIN(a3, a4)))));  
}  
  
inline int min (int a, ...)  
{  
    cerr<< "OUCH  undefined min function (int . . .)";  
    exit (-1);  
}  
  
/*
// rint isn't as universal as its supposed to be :-)
// there should be a #ifndef to do this but . . . what is it ???

    inline double rint(double x)  
    {  
        double a=floor(x);  
        double b=ceil(x);  
        return ( (a-x)<(x-b) ? a :b );  
    }  
*/

#endif  
  
  
