//*****************************************************************************
//      Header file for:
//		screen / real conversion base class
//
//      Written by Stuart Prescott:
//              Written:                21/08/1996
//              Last Modified:          27/10/1996
//
//      Libraries Required:
//              libsx (Dominic Giamapolo - Athena Widget Set)
//
//      System headers:
//              Streamed I/O
//
//      Other Object Classes used:
//              template arrays
//              3D points / vectors / matrices
//              maths extensions
//
//*****************************************************************************

#ifndef SCREEN_INFO_DEF_H
#define SCREEN_INFO_DEF_H 1

extern "C" {
    #include <libsx.h>
}
#include "arraytem.h"
#include "vectors.h"
#include "mathxtra.h"

class ScreenInfo
{
    private:
        point3 origin;              //location of screen origin in WORLD
        point3 delta;               //size of window in WORLD
        XPoint size;                //size of the window in PIXELS

    public:
      //functions
        //set values
        void setscreen (const point3& topleft, const point3& bottomright)
            { origin = topleft;                     //sets top left pixel
              delta = point3(bottomright.x-origin.x, bottomright.y-origin.y);
              return;}              //sets bottom right pixel

        void setpixels (int x, int y) { size.x = x; size.y = y; return;}

        //get values
        point3 getSorigin () {return origin;}
        XPoint getSsize   () {return size;}

        //convert between WORLD and SCREEN co-ords

        int toscreenx (const double& d)
            { return INT((d - origin.x) * size.x / (delta.x)); }

        int toscreeny (const double& d)
            { return INT((d - origin.y) * size.y / (delta.y)); }

        XPoint toscreen (const point3& p)
        {
            XPoint n;
            n.x = toscreenx(p.x);
            n.y = toscreeny(p.y);
            return n;
        }

        double toworldx (const int i)
            { return (delta.x) * i/size.x + origin.x; }

        double toworldy (const int& i)
            { return (delta.y) * i/size.y + origin.y; }

        point3 toworld (const XPoint& p)
            { return point3( toworldx(p.x), toworldy(p.y) ); }

};    //ScreenInfo

#endif
